/**
 * $Id: editor_plugin_src.js 201 2007-02-12 15:56:56Z spocke $
 *
 * @author Moxiecode
 * @copyright Copyright © 2004-2008, Moxiecode Systems AB, All rights reserved.
 */

(function() {
	// Load plugin specific language pack
	tinymce.PluginManager.requireLangPack('formular');

	tinymce.create('tinymce.plugins.FormularPlugin', {
		/**
		 * Initializes the plugin, this will be executed after the plugin has been created.
		 * This call is done before the editor instance has finished it's initialization so use the onInit event
		 * of the editor instance to intercept that event.
		 *
		 * @param {tinymce.Editor} ed Editor instance that the plugin is initialized in.
		 * @param {string} url Absolute URL to where the plugin is located.
		 */
		init : function(ed, url) {

            // Helper function that checks of current selected item is a formular
            // item that could be edited.
            function _nodeNameIsFormularInput(ed, n) {
                return (
                    n.nodeName === 'INPUT'
                    || n.nodeName === 'SELECT'
                    || (n.nodeName === 'IMG' && ed.dom.getAttrib(n, 'src') === 'js/tiny_mce/plugins/formular/img/textarea.jpg')
                    || (n.nodeName === 'IMG' && ed.dom.getAttrib(n, 'src') === 'js/tiny_mce/plugins/formular/img/el_checkbox.gif')
                    || (n.nodeName === 'IMG' && ed.dom.getAttrib(n, 'src') === 'js/tiny_mce/plugins/formular/img/el_checkbox_checked.gif')
                    || (n.nodeName === 'IMG' && ed.dom.getAttrib(n, 'src') === 'js/tiny_mce/plugins/formular/img/el_dropdown.gif')
                    || (n.nodeName === 'IMG' && ed.dom.getAttrib(n, 'src') === 'js/tiny_mce/plugins/formular/img/el_password.gif')
                    || (n.nodeName === 'IMG' && ed.dom.getAttrib(n, 'src') === 'js/tiny_mce/plugins/formular/img/el_radio.gif')
                    || (n.nodeName === 'IMG' && ed.dom.getAttrib(n, 'src') === 'js/tiny_mce/plugins/formular/img/el_radio_checked.gif')
                    || (n.nodeName === 'IMG' && ed.dom.getAttrib(n, 'src') === 'js/tiny_mce/plugins/formular/img/el_reset.gif')
                    || (n.nodeName === 'IMG' && ed.dom.getAttrib(n, 'src') === 'js/tiny_mce/plugins/formular/img/el_submit.gif')
                    || (n.nodeName === 'IMG' && ed.dom.getAttrib(n, 'src') === 'js/tiny_mce/plugins/formular/img/el_text.gif')
                    || (n.nodeName === 'IMG' && ed.dom.getAttrib(n, 'src') === 'js/tiny_mce/plugins/formular/img/el_textarea.gif')
                );
            }

			// Register the command so that it can be invoked by using tinyMCE.activeEditor.execCommand('mceExample');
			ed.addCommand('mceformular', function() {
				ed.windowManager.open({
					file : url + '/popup.php',
					width : 400 + parseInt(ed.getLang('formular.delta_width', 0)),
					height : 250 + parseInt(ed.getLang('formular.delta_height', 0)),
					inline : 1
				}, {
					plugin_url : url, // Plugin absolute URL
					some_custom_arg : 'custom arg' // Custom argument
				});
			});

			// Register example button
			ed.addButton('formular', {
				title : 'formular.desc',
				cmd : 'mceformular',
				image : url + '/img/formular.gif'
			});

			// Add a node change handler, selects the button in the UI when a image is selected
			ed.onNodeChange.add(function(ed, cm, n) {
				var p = ed.dom.getParent(n, 'td,th,caption');
				cm.setDisabled('formular', true);
				cm.setActive('formular', false);
				if (n.nodeName === 'TABLE' || !!p) {
                    cm.setDisabled('formular', false);
					if (_nodeNameIsFormularInput(ed, n)) {
					   cm.setActive('formular', true);
					}
				}
			});
		},

		/**
		 * Creates control instances based in the incomming name. This method is normally not
		 * needed since the addButton method of the tinymce.Editor class is a more easy way of adding buttons
		 * but you sometimes need to create more complex controls like listboxes, split buttons etc then this
		 * method can be used to create those.
		 *
		 * @param {String} n Name of the control to create.
		 * @param {tinymce.ControlManager} cm Control manager to use inorder to create new control.
		 * @return {tinymce.ui.Control} New control instance or null if no control was created.
		 */
		createControl : function(n, cm) {
			return null;
		},

		/**
		 * Returns information about the plugin as a name/value array.
		 * The current keys are longname, author, authorurl, infourl and version.
		 *
		 * @return {Object} Name/value array containing information about the plugin.
		 */
		getInfo : function() {
			return {
				longname : 'Insert formular elements',
				author : 'Daniel Schliebner',
				authorurl : 'http://www.ds-develop.de',
				infourl : 'http://tinymce.moxiecode.com/tinymce/docs/plugin_formular.html',
				version : "1.0"
			};
		}
	});

	// Register plugin
	tinymce.PluginManager.add('formular', tinymce.plugins.FormularPlugin);
})();